#!/usr/bin/perl
# *****************************************************************************
# *                                                                           *
# * Upcoming Plug-In for Movable Type -- greg@eod.com                         *
# *                                                                           *
# *****************************************************************************


# Package *********************************************************************
package MT::Plugin::Upcoming;


# Uses ************************************************************************
use strict;
use warnings;
use MT;
use MT::Template::Context;
eval
{
	require Digest::MD5;
	require File::Spec;
	require Fcntl;
};


# Exports *********************************************************************
our $VERSION = '0.10';


# Statics *********************************************************************
my $Upco_Plug;
my $Upco_Eror;
my $Upco_File;
my $Upco_Time = 60*60;
my %Upco_Meth_Event =
(
	'Id'              => 'id',
	'Name'            => 'name',
	'Description'     => 'description',
	'StartDate'       => 'start_date',
	'StartTime'       => 'start_time',
	'EndDate'         => 'end_date',
	'EndTime'         => 'end_time',
	'Personal'        => 'personal',
	'SelfPromotional' => 'selfpromotional',
	'DatePosted'      => 'date_posted',
);
my %Upco_Meth_Venue =
(
	'Id'              => 'id',
	'Name'            => 'name',
	'Address'         => 'address',
	'City'            => 'city',
	'Zip'             => 'zip',
	'Phone'           => 'phone',
	'Url'             => 'url',
	'Description'     => 'description',
	'UserId'          => 'user_id',
	'Private'         => 'private'
);


# Code ************************************************************************

# Set up ======================================================================
eval
{
	eval { die; };
	$@ =~ /([^\s]+)mt-upcoming.pl/;
	$1 =~ m#/+$#;
	unshift(@INC,"$`/mt-upcoming");

	require WebService::Upcoming;
};
$Upco_Eror = $@;
eval
{
	require MT::Plugin;
        require MT::PluginData;

	my $plug_Data;

	$plug_Data = MT::PluginData->load(
	 {
		'plugin' => 'Upcoming', # Old configuration name
		'key'    => 'configuration'
	 });
	$plug_Data = $plug_Data?$plug_Data->data() || {}:{};
	$Upco_Plug = MT::Plugin->new(
	 {
		# MT 3.x
		'name'             => 'MT-Upcoming',
		'description'      => ($Upco_Eror?
		                    'INSTALLATION ERROR: '.
		                     'WebService::Upcoming did not load! '.
		                     "($Upco_Eror)":
		                    'Adds tags to query Upcoming.org and '.
		                     'display your events.'),
		'doc_link'         => 'http://www.eod.com/code/upcoming/mt/'.
		                     'doc.html',
		'author_name'      => 'Greg Knauss',
		'author_link'      => 'http://www.eod.com/',
		'version'          => $VERSION,
		($MT::VERSION < 3.2)?
		('config_link'     => $Upco_Eror?undef:
		                       'mt-upcoming/mt-upcoming.cgi') :
		('config_template' => \&upco_Template,
		 'settings'        => eval { new MT::PluginSettings([
		                       [ 'upco_user',{ 'Default' =>
		                           $plug_Data->{'upco_user'}} ],
		                       [ 'upco_pass',{ 'Default' =>
		                           $plug_Data->{'upco_pass'}} ],
		                       [ 'upco_keyy',{ 'Default' =>
		                           $plug_Data->{'upco_keyy'}} ] ])} )
	 });
	MT->add_plugin($Upco_Plug);
};
MT::Template::Context->add_container_tag('UpcomingSearch',
	\&upco_Elmt_UpcomingSearch);
MT::Template::Context->add_container_tag('UpcomingWatchlist',
	\&upco_Elmt_UpcomingSearch);
MT::Template::Context->add_tag('UpcomingWatchlistStatus',
	\&upco_Elmt_UpcomingWatchlistStatus);
foreach ('Attend','Watch')
{
	MT::Template::Context->add_conditional_tag(
		'UpcomingWatchlistIfStatus'.$_,
		\&upco_Elmt_UpcomingWatchlistStatus);
}
foreach (keys(%Upco_Meth_Event),'Public')
{
	MT::Template::Context->add_tag('UpcomingEvent'.$_,
		\&upco_Elmt_UpcomingEvent);
	MT::Template::Context->add_conditional_tag('UpcomingEventIf'.$_,
		\&upco_Elmt_UpcomingEvent);
}
foreach (keys(%Upco_Meth_Venue),'Public')
{
	MT::Template::Context->add_tag('UpcomingVenue'.$_,
		\&upco_Elmt_UpcomingVenue);
	MT::Template::Context->add_conditional_tag('UpcomingVenueIf'.$_,
		\&upco_Elmt_UpcomingVenue);
}

# Configuration template ======================================================
sub upco_Template
{
	return <<TMPL;

	<div class="setting">

	 <div class="label"><label for="upco_user">User:</label></div>
	 <div class="field">
	  <p><input type="text" name="upco_user" id="upco_user"
	   value="<TMPL_VAR NAME=UPCO_USER>" size="64" maxlength="32" /><br />
           <MT_TRANS phrase="Enter your Upcoming user name here."></p>
         </div>

	 <div class="label"><label for="upco_pass">Password:</label></div>
	 <div class="field">
	  <p><input type="password" name="upco_pass" id="upco_pass"
	   value="<TMPL_VAR NAME=UPCO_PASS>" size="64" maxlength="32" /><br />
	   <MT_TRANS phrase="Enter your Upcoming password here."></p>
         </div>

	 <div class="label"><label for="upco_keyy">API Key:</label></div>
	 <div class="field">
	  <p><input type="text" name="upco_keyy" id="upco_keyy"
	   value="<TMPL_VAR NAME=UPCO_KEYY>" size="64" maxlength="10" /><br />
	   <MT_TRANS phrase="Enter your Upcoming">
	   <a href="http://upcoming.org/services/api/keygen.php"
	   target="_new">API Key</a> <MT_TRANS phrase="here."></p>
         </div>
        </div>
TMPL
}

# Error message ===============================================================
sub upco_Fail
{
	my $eror_Text;

	$eror_Text = '';
	if ($_[1])
	{
		foreach (keys(%{$_[1]}))
		{
			$eror_Text .=
				($eror_Text?',':'').$_.' => '.$_[1]->{$_};
		}
		$eror_Text = " [Arguments: $eror_Text]";
	}
		
	return $_[0]->error('WebService::Upcoming error: '.
		$_[0]->stash('upco_upco')->err_text().$eror_Text);
}

# Call WebService::Upcoming ===================================================
sub upco_File
{
	my $meth;
	my $args;
	my $file;

	($meth,$args) = @_;
	eval
	{
		my $temp;
		my $md5x;

		$temp = File::Spec->tmpdir();

		# Delete any out-of-date cache files
		if (!$Upco_File)
		{
			my $time;

			foreach (glob(File::Spec->catdir($temp,'upco_*.xml')))
			{
				(undef,undef,undef,undef,undef,undef,undef,
				 undef,undef,$time) = stat($_);
				unlink($_) if ($time+$Upco_Time < time());
			}
			$Upco_File = 1;
		}

		# Generate the cache file name for this query
		$md5x = new Digest::MD5();
		$md5x->add($meth,join(',',%{$args}));
		$file = File::Spec->catfile($temp,
			'upco_'.$md5x->hexdigest().'.xml');
	};
	return $file;
}
sub upco_Call
{
	my $upco;
	my $meth;
	my $args;
	my $file;
	my $rspn;

	($upco,$meth,$args) = @_;
	$file = upco_File($meth,$args);
	if (($file) &&
	    (sysopen(FILE,$file,&Fcntl::O_RDONLY,0600)))
	{
		local $/ = undef;
		$rspn = <FILE>;
		close(FILE);
	}
	if (!defined($rspn))
	{
		$rspn = $upco->query($meth,$args);
		if (defined($rspn) &&
		    ($file)        &&
		    (sysopen(FILE,$file,
		     &Fcntl::O_WRONLY|&Fcntl::O_CREAT|&Fcntl::O_TRUNC,0600)))
		{
			printf FILE ($rspn);
			close(FILE);
		}
	}
	return undef if (!defined($rspn));
	return $upco->parse($meth,$rspn);
}

# Handle <Upcoming(Search|Watchlist)> =========================================
sub upco_Elmt_UpcomingSearch
{
	my $elmt_Cntx;
	my $elmt_Tokn;
	my $elmt_Elmt;
	my $elmt_Args;
	my $elmt_Conf;
	my $elmt_Text;
	my %call_Args;
	my $call_Evnt;
	my $objc_Text;
	my $objc_Indx;
	my $objc_Cont;

	($elmt_Cntx,$elmt_Args) = @_;

	# Get any default parameters
	$elmt_Conf = {};
	if ($Upco_Plug)
	{
		require MT::Request;

		$elmt_Conf = MT::Request->instance->cache('Upcoming');
		if (!$elmt_Conf)
		{
			$elmt_Conf = $Upco_Plug->get_config_hash() || {};
			MT::Request->instance->cache('Upcoming',$elmt_Conf);
		}
	}
	$elmt_Cntx->stash('upco_user',
		$elmt_Args->{'username'} || $elmt_Conf->{'upco_user'});
	$elmt_Cntx->stash('upco_pass',
		$elmt_Args->{'password'} || $elmt_Conf->{'upco_pass'});
	$elmt_Cntx->stash('upco_keyy',
		$elmt_Args->{'api_key'}  || $elmt_Conf->{'upco_keyy'});

	# Build call based on tag
	return $elmt_Cntx->error('WebService::Upcoming did not load! '.
		"($Upco_Eror)") if ($Upco_Eror);
	$elmt_Cntx->stash('upco_upco',new WebService::Upcoming(
		$elmt_Cntx->stash('upco_keyy')));
	$call_Args{'username'} = $elmt_Cntx->stash('upco_user');
	$call_Args{'password'} = $elmt_Cntx->stash('upco_pass');
	$elmt_Elmt = $elmt_Cntx->stash('tag');
	if ($elmt_Elmt eq 'UpcomingWatchlist')
	{
		my $wtch_List;
		my $wtch_Wtch;

		$wtch_List = upco_Call($elmt_Cntx->stash('upco_upco'),
			'watchlist.getList',\%call_Args);
		return upco_Fail($elmt_Cntx,\%call_Args) if
			(!defined($wtch_List));
		foreach (@{$wtch_List})
		{
			$wtch_Wtch->{$_->event_id()} = $_;
		}
		return $elmt_Args->{'none'} || 'No events were found.'
			if (!$wtch_Wtch);
		$elmt_Cntx->stash('upco_wtch',$wtch_Wtch);

		# Expand a watchlist into events
		$call_Evnt = upco_Call($elmt_Cntx->stash('upco_upco'),
			'event.getInfo',
			{
				'username' => $elmt_Cntx->stash('upco_user'),
				'password' => $elmt_Cntx->stash('upco_pass'),
				'event_id' => join(',',keys(
					%{$elmt_Cntx->stash('upco_wtch')}))
			});
	}
	elsif ($elmt_Elmt eq 'UpcomingSearch')
	{
		foreach ('search_text','country_id','state_id','metro_id',
		         'venue_id','min_date','max_date','tags','per_page',
		         'page','sort')
		{
			$call_Args{$_} = $elmt_Args->{$_} if
				($elmt_Args->{$_});
		}
		$call_Evnt = upco_Call($elmt_Cntx->stash('upco_upco'),
			'event.search',\%call_Args);
	}
	return upco_Fail($elmt_Cntx,\%call_Args) if (!defined($call_Evnt));
	return $elmt_Args->{'none'} || 'No events were found.'
		if (!$call_Evnt);
	$elmt_Cntx->stash('upco_evnt',$call_Evnt);

	# Expand an event list into venues
	my $venu_Venu;
	foreach (@{upco_Call($elmt_Cntx->stash('upco_upco'),
		'venue.getInfo',
		{
			'username' => $elmt_Cntx->stash('upco_user'),
			'password' => $elmt_Cntx->stash('upco_pass'),
			'venue_id' => join(',',map($_->venue_id(),
				@{$elmt_Cntx->stash('upco_evnt')}))
		})})
	{
		foreach my $temp_Evnt (@{$elmt_Cntx->stash('upco_evnt')})
		{
			$venu_Venu->{$temp_Evnt->id()} = $_
				if ($temp_Evnt->venue_id() == $_->id());
		}
	}
	$elmt_Cntx->stash('upco_venu',$venu_Venu);

	# Loop through each event
	$elmt_Tokn = $elmt_Cntx->stash('tokens');
	$objc_Indx = 0;
	$objc_Cont = @{$elmt_Cntx->stash('upco_evnt')};
	foreach (sort
	{
		return $a->start_date() cmp $b->start_date();
	} @{$elmt_Cntx->stash('upco_evnt')})
	{
		# Stop and/or skip to the appropriate place
		last if (defined($elmt_Args->{'lastn'}) &&
			($objc_Indx == $elmt_Args->{'lastn'}));
		next if (defined($elmt_Args->{'skip'}) &&
			($objc_Indx <  $elmt_Args->{'skip'}));

		# Keep track of which response we're on for the sub-tags
		$elmt_Cntx->stash('upco_this',$_);

		# Expand any tags in the contents and add it to our result
		$objc_Text = $elmt_Cntx->stash('builder')->build(
			$elmt_Cntx,$elmt_Tokn);
		return $elmt_Cntx->error($elmt_Cntx->errstr) if
			(!defined($objc_Text));
		$elmt_Text .= $objc_Text;

		# Add the appropriate separator to our result
		$objc_Indx++;
		if ($objc_Indx == $objc_Cont)
		{
			# This space intentionally left blank
		}
		elsif ($objc_Indx == $objc_Cont-1)
		{
			$elmt_Text .= $elmt_Args->{'separatorlast'} ||
				$elmt_Args->{'separator'} || '';
		}
		elsif ($objc_Indx == 0)
		{
			$elmt_Text .= $elmt_Args->{'separatorfirst'} ||
				$elmt_Args->{'separator'} || '';
		}
		else
		{
			$elmt_Text .= $elmt_Args->{'separator'} || '';
		}
	}

	# Clean up
	$elmt_Cntx->stash('upco_evnt',undef);
	$elmt_Cntx->stash('upco_wtch',undef);
	$elmt_Cntx->stash('upco_venu',undef);
	$elmt_Cntx->stash('upco_this',undef);

	return $elmt_Text;
}

# Handle <UpcomingWatchlistStatus> ============================================
sub upco_Elmt_UpcomingWatchlistStatus
{
	my $elmt_Cntx;
	my $elmt_Args;
	my $elmt_Elmt;
	my $elmt_Wtch;

	($elmt_Cntx,$elmt_Args) = @_;
	$elmt_Elmt = $elmt_Cntx->stash('tag');
	$elmt_Wtch = $elmt_Cntx->stash('upco_wtch')->{
		$elmt_Cntx->stash('upco_this')->id()};

	# Handle no data
	return '[&lt;'.$elmt_Elmt.'&gt; may only be used inside '.
		'&lt;UpcomingWatchlist&gt;!]' if
		(!$elmt_Cntx->stash('upco_wtch'));
	return '[No data available for this &lt;'.$elmt_Elmt.'&gt;!]' if
		(!$elmt_Wtch);

	# Handle conditional tags
	if ($elmt_Elmt =~ /^UpcomingWatchlistIfStatus/)
	{
		return ($elmt_Wtch->status() eq 'attend')?1:0 if
			($' eq 'Attend');
		return ($elmt_Wtch->status() eq 'watch')?1:0;
	}

	# Handle everything else
	return $elmt_Wtch->status();
}

# Handle <UpcomingEvent*> =====================================================
sub upco_Elmt_UpcomingEvent
{
	my $elmt_Cntx;
	my $elmt_Args;
	my $elmt_Elmt;
	my $elmt_Evnt;
	my $elmt_Attr;
	my $elmt_Meth;

	($elmt_Cntx,$elmt_Args) = @_;

	# Find out which tag we're dealing with and the associated
	# WebService::Upcoming method
	$elmt_Elmt = $elmt_Cntx->stash('tag');
	$elmt_Elmt =~ /^UpcomingEvent(If|)/;
	$elmt_Attr = $';
	$elmt_Meth = $Upco_Meth_Event{$elmt_Attr};
	$elmt_Evnt = $elmt_Cntx->stash('upco_this');

	# Handle no data
	return '[&lt;'.$elmt_Elmt.'&gt; may only be used inside '.
		'&lt;UpcomingWatchlist&gt; or &lt;UpcomingSearch&gt;!]' if
		(!$elmt_Cntx->stash('upco_evnt'));
	return '[No data available for this &lt;'.$elmt_Elmt.'&gt;!]' if
		(!$elmt_Evnt);

	# Handle non-method tags
	return $elmt_Evnt->personal()?0:1 if
		($elmt_Elmt =~ /^UpcomingEvent(If|)Public$/);

	# Handle conditional tags
	if ($elmt_Elmt =~ /^UpcomingEventIf/)
	{
		my $cond_Text;

		# Get the text of the response
		$cond_Text = $elmt_Evnt->$elmt_Meth();

		# Fix up a wonky response from the Upcoming API 1.0
		$cond_Text = '' if (
			($elmt_Elmt =~ /Date/) &&
			($cond_Text eq '0000-00-00'));

		return ($cond_Text)?1:0;
	}

	# Handle date/time tags
	if (($elmt_Attr =~ /Date/) || ($elmt_Attr =~ /Time/))
	{
		my %date_Args;
		my $date_Met1;
		my $date_Met2;
		my $date_Date;
		my $date_Time;

		# Convert Dates and Times to the same thing for standard
		# MT date/time formatting
		%date_Args = %{$elmt_Args};
		$date_Met1 = 'date_posted';
		if ($elmt_Attr =~ /^Start/)
		{
			$date_Met1 = 'start_date';
			$date_Met2 = 'start_time';
		}
		elsif ($elmt_Attr =~ /^End/)
		{
			$date_Met1 = 'end_date';
			$date_Met2 = 'end_time';
		}

		# Read each part and normalize them
		$date_Date = $elmt_Evnt->$date_Met1();
		$date_Date =~ s/[^0-9]//g;
		$date_Time = $elmt_Evnt->$date_Met2() || '000000';
		$date_Args{'format'} = '%B %e, %Y' if
			((!$date_Time) && (!defined($date_Args{'format'})));
		$date_Time = sprintf('%02d%02d%02d',
			$date_Time =~ /(\d\d)[^\d]?(\d\d)[^\d]?(\d\d)/);
		$date_Args{'ts'} = $date_Date.$date_Time;

		# Use the standard MT formatting
		return MT::Template::Context::_hdlr_date($elmt_Cntx,
			\%date_Args);
	}

	# Handle everything else
	return $elmt_Evnt->$elmt_Meth();
}

# Handle <UpcomingVenue*> =====================================================
sub upco_Elmt_UpcomingVenue
{
	my $elmt_Cntx;
	my $elmt_Args;
	my $elmt_Elmt;
	my $elmt_Attr;
	my $elmt_Meth;
	my $elmt_Venu;

	($elmt_Cntx,$elmt_Args) = @_;

	# Find out which tag we're dealing with and the associated
	# WebService::Upcoming method
	$elmt_Elmt = $elmt_Cntx->stash('tag');
	$elmt_Elmt =~ /^UpcomingVenue(If|)/;
	$elmt_Attr = $';
	$elmt_Meth = $Upco_Meth_Venue{$elmt_Attr};
	$elmt_Venu = $elmt_Cntx->stash('upco_venu')->{
		$elmt_Cntx->stash('upco_this')->id()};

	# Handle no data
	return '[&lt;'.$elmt_Elmt.'&gt; may only be used inside '.
		'&lt;UpcomingWatchlist&gt; or &lt;UpcomingSearch&gt;]' if
		(!$elmt_Cntx->stash('upco_venu'));
	return '[No data available for this &lt;'.$elmt_Elmt.'&gt;!]' if
		(!$elmt_Venu);

	# Handle non-method tags
	return $elmt_Venu->private()?0:1 if
		($elmt_Elmt =~ /^UpcomingVenue(If|)Public$/);

	# Handle conditional tags
	return ($elmt_Venu->$elmt_Meth())?1:0 if
		($elmt_Elmt =~ /^UpcomingVenueIf/);

	# Handle everything else
	return $elmt_Venu->$elmt_Meth();
}
1;
